<?php
/*
NOTICE OF LICENSE

This source file is subject to the NekloEULA that is bundled with this package in the file ICENSE.txt.

It is also available through the world-wide-web at this URL: http://store.neklo.com/LICENSE.txt

Copyright (c)  Neklo (http://store.neklo.com/)
*/


namespace Neklo\Scheduler\Block\Adminhtml\Schedule;

use Neklo\Scheduler\Model\ResourceModel\Schedule\Timeline\CollectionFactory as TimelineCollectionFactory;
use Neklo\Scheduler\Model\Schedule\Filters;

class Timeline extends \Magento\Backend\Block\Template
{
    /**
     * @var TimelineCollectionFactory
     */
    private $timelineCollectionFactory;

    /**
     * @var Filters
     */
    private $filters;

    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        TimelineCollectionFactory $collection,
        Filters $filters,
        array $data = []
    ) {
        $this->timelineCollectionFactory = $collection;
        $this->filters = $filters;
        parent::__construct($context, $data);
    }

    /**
     * @return \Neklo\Scheduler\Model\ResourceModel\Schedule\Timeline\Collection
     */
    public function getJobs()
    {
        $jobs = $this->timelineCollectionFactory->create();
        $jobs->setOrder('job_code', 'ASC');
        $this->applyFilters($jobs);
        return $jobs;
    }

    /**
     *
     * @param \Neklo\Scheduler\Model\ResourceModel\Schedule\Timeline\Collection $collection
     */
    protected function applyFilters(
        \Neklo\Scheduler\Model\ResourceModel\Schedule\Timeline\Collection $collection
    ) {
        $filters = $this->filters;
        $col = $collection;

        $filterData = $filters->getData();
        foreach ($filterData as $name => $data) {
            switch (true) {
                case $name != 'show_jobs'://dates
                    if (isset($data['date_from']) && $data['date_from']) {
                        $this->setData('start_time', strtotime($data['date_from']));
                        $col->addFieldToFilter(
                            $name . '_at',
                            ['gteq' => date("Y-m-d H:i:s", $this->getData('start_time'))]
                        );
                    }

                    if (isset($data['date_to']) && $data['date_to']) {
                        $this->setData('end_time', strtotime($data['date_to']));
                        $col->addFieldToFilter(
                            $name . '_at',
                            ['lteq' => date("Y-m-d H:i:s", $this->getData('end_time'))]
                        );
                    }

                    if ((!isset($data['date_from']) || $data['date_from'] == '')
                        && (!isset($data['date_to']) || $data['date_to'] == '')) {
                        $this->setData('start_time', time() - (60 * 60 * 24 * 7));//2 months
                        $col->addFieldToFilter(
                            $name . '_at',
                            ['gteq' => date("Y-m-d H:i:s", $this->getData('start_time'))]
                        );
                    } elseif (!isset($data['date_from']) && isset($data['date_to'])) {
                        $time = strtotime($data['date_to']) - (60 * 60 * 24 * 7);//2 months
                        $this->setData('start_time', $time);
                        $col->addFieldToFilter(
                            $name . '_at',
                            ['gteq' => date("Y-m-d H:i:s", $this->getData('start_time'))]
                        );
                    }

                    break;
                default://job names
                    if (!empty($data)) {
                        $col->addFieldToFilter('job_code', ['in' => $data]);
                    }

                    break;
            }
        }
    }

    public function getDataSource($collection)
    {
        $ret = [];
        foreach ($collection->getItems() as $item) {
            $ret[$item->getId() . ''] = $item->toArray();
        }
        return json_encode($ret, JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_UNESCAPED_UNICODE);
    }

    public function getStartTime($collection)
    {
        if (!$this->getData('start_time')) {
            $coll = $collection;
            $start = PHP_INT_MAX;
            foreach ($coll as $rec) {
                if ($rec->getScheduledAt()) {
                    $tmp = strtotime($rec->getScheduledAt());
                    if ($tmp < $start) {
                        $start = $tmp;
                    }
                }
                if ($rec->getExecutedAt()) {
                    $tmp = strtotime($rec->getExecutedAt());
                    if ($tmp < $start) {
                        $start = $tmp;
                    }
                }
                if ($rec->getFinishedAt()) {
                    $tmp = strtotime($rec->getFinishedAt());
                    if ($tmp < $start) {
                        $start = $tmp;
                    }
                }
            }

            $this->setData('start_time', $start);
        }

        return $this->getData('start_time');
    }

    public function getEndTime($collection)
    {
        if (!$this->getData('end_time')) {
            $coll = $collection;
            $start = 0;
            foreach ($coll as $rec) {
                if ($rec->getScheduledAt()) {
                    $tmp = strtotime($rec->getScheduledAt());
                    if ($tmp > $start) {
                        $start = $tmp;
                    }
                }

                if ($rec->getExecutedAt()) {
                    $tmp = strtotime($rec->getExecutedAt());
                    if ($tmp > $start) {
                        $start = $tmp;
                    }
                }

                if ($rec->getFinishedAt()) {
                    $tmp = strtotime($rec->getFinishedAt());
                    if ($tmp > $start) {
                        $start = $tmp;
                    }
                }
            }

            $this->setData('end_time', $start);
        }

        return $this->getData('end_time');
    }

    public function decorateTime($time, $format)
    {
        $time *= 1;
        return date($format, $time);
    }
}
