<?php
/*
NOTICE OF LICENSE

This source file is subject to the NekloEULA that is bundled with this package in the file ICENSE.txt.

It is also available through the world-wide-web at this URL: http://store.neklo.com/LICENSE.txt

Copyright (c)  Neklo (http://store.neklo.com/)
*/


namespace Neklo\Scheduler\Block\Adminhtml\Schedule\Timeline;

use Neklo\Scheduler\Model\Schedule\Filters;

/**
 * @method \Neklo\Scheduler\Model\Schedule\Filter getFilters()
 */
class Filter extends \Magento\Backend\Block\Widget\Form\Generic
{
    /**
     * @var \Neklo\Scheduler\Helper\Job
     */
    private $jobHelper;

    /**
     * @var Filters
     */
    private $filterData;

    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Data\FormFactory $formFactory,
        \Neklo\Scheduler\Helper\Job $jobHelper,
        Filters $filters,
        array $data = []
    ) {
        parent::__construct($context, $registry, $formFactory, $data);
        $this->jobHelper = $jobHelper;
        $this->filterData = $filters;
    }

    protected function _construct()
    {
        parent::_construct();
        $this->setId('timeline_filter_form');
        $this->setTitle(__('Filters'));
    }

    public function setFilter($name, $type = '', $label = '')
    {
        if (!isset($this->_data['filters'])) {
            $this->_data['filters'] = [];
        }
        if (!isset($this->_data['filters'][$name])) {
            $this->_data['filters'][$name] = [
                'name'  => $name,
                'label' => $label,
                'type'  => $type,
            ];
        } else {
            throw new \Exception(__("Filter '%1' allready defined", $name));
        }
    }

    protected function _prepareLayout()
    {
        $this->getToolbar()->addChild(
            'submit_button',
            'Magento\Backend\Block\Widget\Button',
            [
                'label'   => __('Submit'),
                'class'   => 'primary',
                'type'    => 'submit',
                'onclick' => "jQuery('#{$this->getId()}').submit()",
            ]
        );

        $this->getToolbar()->addChild(
            'toggle_button',
            'Magento\Backend\Block\Widget\Button',
            [
                'label'   => __('Show/Hide'),
                'title'   => __('Show/Hide filters'),
                'class'   => 'default',
                'onclick' => "jQuery('.timeline .admin__data-grid-header').toggle('slow')",
            ]
        );

        $clearUrl = $this->getUrl(
            '*/*/*',
            [
                '_current'       => true,
                'timeline_clear' => 1,
            ]
        );

        $this->getToolbar()->addChild(
            'clear_button',
            'Magento\Backend\Block\Widget\Button',
            [
                'label'   => __('Clear'),
                'onclick' => "setLocation('{$clearUrl}')"
            ]
        );

        return parent::_prepareLayout();
    }

    /**
     * Prepare form
     *
     * @return $this
     */
    protected function _prepareForm()
    {
        /** @var \Magento\Framework\Data\Form $form */
        $form = $this->_formFactory->create(
            [
                'data' => [
                    'id'     => $this->getId(),
                    'action' => $this->getUrl('*/*/*'),
                    'method' => 'post',
                ]
            ]
        );

        $form->setHtmlIdPrefix('block_');

        $dateFieldset = $form->addFieldset('date_fieldset', ['admin__data-grid-filters']);
        $jobFieldset = $form->addFieldset(
            'job_fieldset',
            ['class' => 'fieldset-wide']
        );
        foreach ($this->getFilters() as $name => $cfg) {
            $fieldCfg = [
                'name'  => Filters::FILTER . '[' . $name . ']',
                'label' => __($cfg['label']),
                'title' => __($cfg['label']),
            ];
            switch ($cfg['type']) {
                case 'date':
                    $field = $dateFieldset->addFieldset(
                        'date_fieldset_' . $name,
                        [
                            'legend' => __($cfg['label']),
                            'class'  => 'admin__form-field'
                        ]
                    );

                    $fieldCfg['date_format'] = 'yyyy-MM-dd';
                    $fieldCfg['time_format'] = 'hh:mm:ss';
                    $fieldCfg['show_time'] = true;
                    $fieldCfg['name'] = Filters::FILTER . '[' . $name . '][date_from]';
                    $fieldCfg['label'] = 'From';
                    $fieldCfg['title'] = 'From';
                    $field->addField($name . '_date_from', $cfg['type'], $fieldCfg);
                    $fieldCfg['name'] = Filters::FILTER . '[' . $name . '][date_to]';
                    $fieldCfg['label'] = 'To';
                    $fieldCfg['title'] = 'To';
                    $field->addField($name . '_date_to', $cfg['type'], $fieldCfg);
                    break;
                case 'multiselect':
                    $values = [];
                    foreach ($this->jobHelper->getJobCodes() as $code) {
                        $values[] = ['value' => $code, 'label' => $code];
                    }

                    $fieldCfg['values'] = $values;
                    $jobFieldset->addField($name, $cfg['type'], $fieldCfg);
                    break;
            }
        }

        $model = $this->filterData->getData();
        $this->_logger->debug(print_r($model, 1));
        $form->setValues($model);
        $form->setUseContainer(true);
        $this->setForm($form);

        return parent::_prepareForm();
    }
}
