<?php
/*
NOTICE OF LICENSE

This source file is subject to the NekloEULA that is bundled with this package in the file ICENSE.txt.

It is also available through the world-wide-web at this URL: http://store.neklo.com/LICENSE.txt

Copyright (c)  Neklo (http://store.neklo.com/)
*/


namespace Neklo\Scheduler\Helper;

use Magento\Cron\Model\ConfigInterface;
use Magento\Framework\App\Helper\AbstractHelper;

class Job extends AbstractHelper implements ConfigInterface
{
    /**
     * @var array
     */
    private $jobs;

    /**
     * Cron config data
     *
     * @var \Magento\Cron\Model\Config\Data
     */
    private $configData;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Neklo\Scheduler\Model\Config $configData
    ) {
        $this->configData = $configData;
        parent::__construct($context);
    }

    public function getJobCodes($group = null)
    {
        $codes = [];
        $jobs = $this->getJobs($group);
        if (null === $group) {
            foreach ($jobs as $groupJobs) {
                $codes = array_merge($codes, array_keys($groupJobs));
            }
        } else {
            $codes = array_keys($jobs);
        }

        // Sort
        sort($codes);

        return $codes;
    }

    /**
     * @param null $group
     * @return array|mixed
     * @throws \Exception
     */
    public function getJobs($group = null)
    {
        if (null === $this->jobs) {
            $this->jobs = $this->configData->getJobs();
        }

        if (null === $group) {
            return $this->jobs;
        } else {
            if (isset($this->jobs[$group])) {
                return $this->jobs[$group];
            } else {
                throw new \Exception(__("Job group '%1' does not exist", $group));
            }
        }
    }

    public function getJobNameOptions($group = null)
    {
        $names = $this->getJobCodes($group);
        $ret = [];
        foreach ($names as $name) {
            $ret[] = ['value' => $name, 'label' => $name];
        }

        return $ret;
    }

    /**
     * @param $schedule
     * @return bool
     */
    public function validateSchedule($schedule)
    {
        $result = preg_match(
//            "/(\*|[0-5]?[0-9]|\*\/[0-9]+)\s+"
//            ."(\*|1?[0-9]|2[0-3]|\*\/[0-9]+)\s+"
//            ."(\*|[1-2]?[0-9]|3[0-1]|\*\/[0-9]+)\s+"
//            ."(\*|[0-9]|1[0-2]|\*\/[0-9]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec)\s+"
//            ."(\*\/[0-9]+|\*|[0-7]|sun|mon|tue|wed|thu|fri|sat)\s*"
//            ."(\*\/[0-9]+|\*|[0-9]+)?/i"
            '/^(\*|((\*\/)?[1-5]‌​?[0-9])) ' .
            '(\*|((\*\/)?[1-5]?[0‌​-9])) ' .
            '(\*|((\*\/)?(1?[0-9]‌​|2[0-3]))) ' .
            '(\*|((\*\/)?([1-9]|[‌​12][0-9]|3[0-1]))) ' .
            '(\*|((\*\/)?([1-9]|1‌​[0-2]))) ' .
            '(\*|((\*\/)?[0-6]))$‌​/i',
            $schedule
        );
        return $result === 1;
    }

    public function isEnabled($name)
    {
        return true;
    }
}
